function [varargout] = crsLiveTrackGetBufferedEyePositions(varargin)
% CRSLIVETRACKGETBUFFEREDEYEPOSITIONS returns buffered eye positions.
% ----------------------------------------------------------------------------
% crsLiveTrackGetBufferedEyePositions
% ==========================
% This function returns eye position data from the buffer that is created and
% maintained by the LiveTrack library. Eye positions that are returned by this
% function are (by default) removed from the data buffer,
% so subsequent calls will not return duplicate data.
%
% If there are no points available in the buffer, a structure filled with zeros
% is returned, together with a non-zero (-1) ErrorCode. NOTE: In order to
% determine if the subject is fixating, eye position data is delayed by
% the user-specifiable fixation period, defined using the vetSetFixationPeriod
% function.
%
% This function has two optional parameters, a Boolean/logical value:
% removeFromBuffer, and a numeric (integer) value: maximumPoints. If the
% removeFromBuffer parameter is set to False, then data points are left in the
% VET buffer, allowing them to be retrieved again by a second call. If the
% removeFromBuffer parameter is set to True, then data points are deleted
% from the buffer after they are read. The maximumPoints parameter allows
% you to limit the number of points returned by this function. This is useful
% if the buffer is large and you can only deal with a limited number of points
% at any one time.
%
% MATLAB Syntax
% =============
% Data = crsLiveTrackGetBufferedEyePositions;
% Data = crsLiveTrackGetBufferedEyePositions(removeFromBuffer);
% Data = crsLiveTrackGetBufferedEyePositions(maximumPoints);
% Data = crsLiveTrackGetBufferedEyePositions(maximumPoints,removeFromBuffer);
%
% Parameters
% ==========
% maximumPoints    - (Optional parameter) A scalar integer indicating the
%                    maximum number of points to return to MATLAB.
%
% removeFromBuffer - (Optional parameter) A scalar logical controlling the
%                    removal of returned eye positions from the buffer.
%
% Return Values
% =============
% Data - A MATLAB structure containing the VET eye position data.
%  Data.timeStamps         - Milliseconds since tracking started.
%  Data.digitalIO          - Integer digital IO value.
%  Data.fixation           - 1 if eyes are fixating, 0 otherwise.
%
%  Data.mmPositions        - Direction of gaze projected onto stimulus display, in mm: [X,Y].
%  Data.fickPositions      - Eye rotation in Fick coordinates: [Longitude, Latitude].
%  Data.helmholtzPositions - Eye rotation in Helmholtz coordinates: [Elevation, Azimuth].
%  Data.pupilDiameter      - Pupil diameter in mm.
%  Data.tracked            - 1 if frame tracked, 0 otherwise.
%  Data.dropped            - 1 if frame dropped, 0 otherwise.
%  Data.calibrated         - 1 if eye calibrated, 0 otherwise.
%  Data.region             - Current region.
%  Data.pupilPositions     - when streaming raw data, the location of the centre of the pupil in camera pixels {X, Y]
%  Data.glintPositions     - when streaming raw data, the location of the centre of the glint in camera pixels {X, Y]
%
% If binocular, the above will be left eye and right eye will be:
%  Data.mmPositionsRight        - Direction of gaze projected onto stimulus display, in mm: [X,Y].
%  Data.fickPositionsRight      - Eye rotation in Fick coordinates: [Longitude, Latitude].
%  Data.helmholtzPositionsRight - Eye rotation in Helmholtz coordinates: [Elevation, Azimuth].
%  Data.pupilDiameterRight      - Pupil diameter in mm.
%  Data.trackedRight            - 1 if frame tracked, 0 otherwise.
%  Data.droppedRight            - 1 if frame dropped, 0 otherwise.
%  Data.calibratedRight         - 1 if eye calibrated, 0 otherwise.
%  Data.regionRight             - Current region.
%  Data.pupilPositionsRight     - when streaming raw data, the location of the centre of the pupil in camera pixels {X, Y]
%  Data.glintPositionsRight     - when streaming raw data, the location of the centre of the glint in camera pixels {X, Y]
%
% HISTORY:
% 2018/01 JT updated to handle empty data buffer
% ----------------------------------------------------------------------------
global CRS;

% HANDLE DIFFERENT INPUT PARAMETERS
% =================================

% NO PARAMETERS
if(nargin == 0)
    remove        =  true;
    maximumPoints = -1;
% ONE LOGICAL PARAMETER (=remove)
elseif(nargin==1 && islogical(varargin{1}))
    remove        = varargin{1};
    maximumPoints = -1;
% ONE NUMERIC PARAMETER (=maximumPoints)
elseif(nargin == 1 && isnumeric(varargin{1}))
    remove        =  1;
    maximumPoints = varargin{1};
% TWO PARAMETERS
elseif(nargin == 2 && isnumeric(varargin{1}) && islogical(varargin{2}))
    maximumPoints = varargin{1};
    remove        = varargin{2};
else
    error('Unrecognised input');
end

% % Translate between a logical 'remove' and some constant values.
% if(remove == 1); removeConstant = CRS.geptGetAndRemoveBuffered;
% else            removeConstant = CRS.geptGetBuffered;          end;

% Always get samples from the beginning of the buffer. Set cmd to 0 to get
% samples from the end of the buffer
cmd = 1; 

if crsLiveTrackGetResultsCount>0
    % Call the library function
    [timeStamps, ~,...
            pupilPositionX, pupilPositionY, glintPositionX, glintPositionY, pupilDiameterL, ~,...
            screenPositionXmmL, screenPositionYmmL, viewDist, helmholtz_AzimuthL, helmholtz_ElevationL, fick_LongitudeL,...
            fick_LatitudeL, trackedL, ~, calibratedL, droppedL, regionL,...
            pupilPositionXR, pupilPositionYR, glintPositionXR, glintPositionYR, pupilDiameterR, ~,...
            screenPositionXmmR, screenPositionYmmR, viewDistR,helmholtz_AzimuthR, helmholtz_ElevationR, fick_LongitudeR,...
            fick_LatitudeR, trackedR, ~, calibratedR, droppedR, regionR,...
            digitalIO, FixationDetected, ErrorCode]...
            = crsLiveTrack(CRS.mexGetResults, maximumPoints, remove, cmd);

    % Copy the data to a structure
    data.timeStamps         = timeStamps(:);
    data.digitalIO          = digitalIO(:);
    data.fixation           = FixationDetected(:);

    data.mmPositions        = [screenPositionXmmL(:),  screenPositionYmmL(:) viewDist(:)];
    data.fickPositions      = [fick_LongitudeL(:),     fick_LatitudeL(:)   ];
    data.helmholtzPositions = [helmholtz_ElevationL(:),helmholtz_AzimuthL(:)];
    data.pupilDiameter      = pupilDiameterL(:);
    data.tracked            = trackedL(:);
    data.dropped            = droppedL(:);
    data.calibrated         = calibratedL(:);
    data.region             = regionL(:);
    data.pupilPositions     = [pupilPositionX(:),  pupilPositionY(:)]; 
    data.glintPositions     = [glintPositionX(:),  glintPositionY(:)]; 

    data.mmPositionsRight          = [screenPositionXmmR(:),  screenPositionYmmR(:) viewDistR(:)];
    data.fickPositionsRight        = [fick_LongitudeR(:),     fick_LatitudeR(:)   ];
    data.helmholtzPositionsRight   = [helmholtz_ElevationR(:),helmholtz_AzimuthR(:)];
    data.pupilDiameterRight        = pupilDiameterR(:);
    data.trackedRight              = trackedR(:);
    data.droppedRight              = droppedR(:);
    data.calibratedRight           = calibratedR(:);
    data.regionRight               = regionR(:);
    data.pupilPositionsRight       = [pupilPositionXR(:),  pupilPositionYR(:)]; 
    data.glintPositionsRight       = [glintPositionXR(:),  glintPositionYR(:)]; 
else 
    % Copy the empty space to a structure
    data.timeStamps         = 0;
    data.digitalIO          = 0;
    data.fixation           = 0;

    data.mmPositions        = [0, 0, 0];
    data.fickPositions      = [0, 0 ];
    data.helmholtzPositions = [0,0];
    data.pupilDiameter      = 0;
    data.tracked            = 0;
    data.dropped            = 0;
    data.calibrated         = 0;
    data.region             = 0;
    data.pupilPositions     = [0,0]; 
    data.glintPositions     = [0,  0]; 

    data.mmPositionsRight          = [0, 0, 0];
    data.fickPositionsRight        = [0, 0 ];
    data.helmholtzPositionsRight   = [0,0];
    data.pupilDiameterRight        = 0;
    data.trackedRight              = 0;
    data.droppedRight              = 0;
    data.calibratedRight           = 0;
    data.regionRight               = 0;
    data.pupilPositionsRight       = [0,  0]; 
    data.glintPositionsRight       = [0,  0]; 
    ErrorCode = -1;
end

if nargout ==  1
    varargout{1} = data;
elseif nargout ==  2
    varargout{1} = data;
    varargout{2} = ErrorCode;
% Undocumented (legacy) usage - only returns monocular data
elseif nargout ==  11
    varargout{1}  = data.timeStamps         ;
    varargout{2}  = data.tracked            ;
    varargout{3}  = data.calibrated         ;
    varargout{4}  = data.pupilDiameter      ;
    varargout{5}  = data.mmPositions        ;
    varargout{6}  = data.fickPositions      ;
    varargout{7}  = data.helmholtzPositions ;
    varargout{8}  = data.region             ;
    varargout{9}  = data.fixation           ;
    varargout{10} = data.digitalIO          ;
    varargout{11} = data.dropped            ;
% Undocumented (legacy) usage - only returns monocular data
elseif nargout ==  12
    varargout{1}  = data.timeStamps         ;
    varargout{2}  = data.tracked            ;
    varargout{3}  = data.calibrated         ;
    varargout{4}  = data.pupilDiameter      ;
    varargout{5}  = data.mmPositions        ;
    varargout{6}  = data.fickPositions      ;
    varargout{7}  = data.helmholtzPositions ;
    varargout{8}  = data.region             ;
    varargout{9}  = data.fixation           ;
    varargout{10} = data.digitalIO          ;
    varargout{11} = data.dropped            ;
    varargout{13} = data.ErrorCode          ;
else
    error('Unrecognised number of output (LHS) arguments.');
end