function varargout = crsLiveTrackGetLatestEyePosition(varargin)
% CRSLIVETRACKGETLATESTEYEPOSITION returns the latest eye position.
% ----------------------------------------------------------------------------
% crsLiveTrackGetLatestEyePosition
% =======================
% This function returns the most recent eye position analysed by the
% LiveTrack library. This data point is NOT removed from the data
% buffer that the LiveTrack library maintains. (I.e. you can get the
% same results later using crsLiveTrackGetBufferedEyePositions). It is also
% possible to get more than one sample by specifying the input parameter
% maximumPoints.
%
% If there are no points available in the buffer, a MATLAB structure filled
% with zeros is returned, together with a non-zero (-1) ErrorCode. This may
% happen if you attept to read data soon after tracking starts, as eye position
% data can be delayed by the user-specifiable fixation period, defined using
% the crsLiveTrackSetFixationPeriod command.
%
% MATLAB Syntax
% =============
%  Data            = crsLiveTrackGetLatestEyePosition;
% [Data,ErrorCode] = crsLiveTrackGetLatestEyePosition;
%  Data            = crsLiveTrackGetLatestEyePosition(maximumPoints);
% [Data,ErrorCode] = crsLiveTrackGetLatestEyePosition(maximumPoints);
%
% Return Values
% =============
% Data - A MATLAB structure containing the VET eye position data.
%  Data.timeStamps         - Milliseconds since tracking started.
%  Data.digitalIO          - Integer digital IO value.
%  Data.fixation           - 1 if eyes are fixating, 0 otherwise.
%
%  Data.mmPositions        - Direction of gaze projected onto stimulus display, in mm: [X,Y].
%  Data.fickPositions      - Eye rotation in Fick coordinates: [Longitude, Latitude].
%  Data.helmholtzPositions - Eye rotation in Helmholtz coordinates: [Elevation, Azimuth].
%  Data.pupilDiameter      - Pupil diameter in mm.
%  Data.tracked            - 1 if frame tracked, 0 otherwise.
%  Data.dropped            - 1 if frame dropped, 0 otherwise.
%  Data.calibrated         - 1 if eye calibrated, 0 otherwise.
%  Data.region             - Current region.
%  Data.pupilPositions     - when streaming raw data, the location of the centre of the pupil in camera pixels {X, Y]
%  Data.glintPositions     - when streaming raw data, the location of the centre of the glint in camera pixels {X, Y]
%
% If binocular, the above will be left eye and right eye will be:
%  Data.mmPositionsRight        - Direction of gaze projected onto stimulus display, in mm: [X,Y].
%  Data.fickPositionsRight      - Eye rotation in Fick coordinates: [Longitude, Latitude].
%  Data.helmholtzPositionsRight - Eye rotation in Helmholtz coordinates: [Elevation, Azimuth].
%  Data.pupilDiameterRight      - Pupil diameter in mm.
%  Data.trackedRight            - 1 if frame tracked, 0 otherwise.
%  Data.droppedRight            - 1 if frame dropped, 0 otherwise.
%  Data.calibratedRight         - 1 if eye calibrated, 0 otherwise.
%  Data.regionRight             - Current region.
%  Data.pupilPositionsRight     - when streaming raw data, the location of the centre of the pupil in camera pixels {X, Y]
%  Data.glintPositionsRight     - when streaming raw data, the location of the centre of the glint in camera pixels {X, Y]
%
%
% Example
% =======
% Since data can be delayed by the fixation period, data may not be ready the
% first time you attempt to read the latest eye position. In that case, the
% following section of code should allow you to pause until valid positions
% can be read:
%
% ...
% crsLiveTrackStartTracking;
% [Data,ErrorCode] = crsLiveTrackGetLatestEyePosition;
% while ErrorCode~=0; [Data,ErrorCode] = crsLiveTrackGetLatestEyePosition; pause(0.1); end;
%
% ----------------------------------------------------------------------------

% We keep all our constants in one place for neatness.
global CRS;

% if no input parameters were given
if(nargin == 0)
    maximumPoints = 1;
else
    maximumPoints = varargin{1};
end

if maximumPoints<1
    maximumPoints = 1;
end

% Always retain the samples in the buffer i.e. do not remove. Set remove to
% 1 to delete samples in the buffer.
remove = 0;

% Always get samples from the end of the buffer. Set cmd to a to get
% samples from the start of the buffer
cmd = 0; 

% Call the library function
[timeStamps, ~,...
        pupilPositionX, pupilPositionY, glintPositionX, glintPositionY, pupilDiameterL, ~,...
        screenPositionXmmL, screenPositionYmmL, viewDist, helmholtz_AzimuthL, helmholtz_ElevationL, fick_LongitudeL,...
        fick_LatitudeL, trackedL, ~, calibratedL, droppedL, regionL,...
        pupilPositionXR, pupilPositionYR, glintPositionXR, glintPositionYR, pupilDiameterR, ~,...
        screenPositionXmmR, screenPositionYmmR, viewDistR,helmholtz_AzimuthR, helmholtz_ElevationR, fick_LongitudeR,...
        fick_LatitudeR, trackedR, ~, calibratedR, droppedR, regionR,...
        digitalIO, FixationDetected, ErrorCode]...
        = crsLiveTrack(CRS.mexGetResults, maximumPoints, remove, cmd);
  
% Copy the data to a structure
data.timeStamps         = timeStamps(:);
data.digitalIO          = digitalIO(:);
data.fixation           = FixationDetected(:);

data.mmPositions        = [screenPositionXmmL(:),  screenPositionYmmL(:) viewDist(:)];
data.fickPositions      = [fick_LongitudeL(:),     fick_LatitudeL(:)   ];
data.helmholtzPositions = [helmholtz_ElevationL(:), helmholtz_AzimuthL(:)];
data.pupilDiameter      = pupilDiameterL(:);
data.tracked            = trackedL(:);
data.dropped            = droppedL(:);
data.calibrated         = calibratedL(:);
data.region             = regionL(:);
data.pupilPositions     = [pupilPositionX(:),  pupilPositionY(:)]; 
data.glintPositions     = [glintPositionX(:),  glintPositionY(:)]; 

data.mmPositionsRight          = [screenPositionXmmR(:),  screenPositionYmmR(:) viewDistR(:)];
data.fickPositionsRight        = [fick_LongitudeR(:),     fick_LatitudeR(:)   ];
data.helmholtzPositionsRight   = [helmholtz_ElevationR(:), helmholtz_AzimuthR(:)];
data.pupilDiameterRight        = pupilDiameterR(:);
data.trackedRight              = trackedR(:);
data.droppedRight              = droppedR(:);
data.calibratedRight           = calibratedR(:);
data.regionRight               = regionR(:);
data.pupilPositionsRight       = [pupilPositionXR(:),  pupilPositionYR(:)]; 
data.glintPositionsRight       = [glintPositionXR(:),  glintPositionYR(:)]; 


if nargout ==  1
    varargout{1} = data;
elseif nargout ==  2
    varargout{1} = data;
    varargout{2} = ErrorCode;
% Undocumented (legacy) usage - only returns monocular data
elseif nargout ==  11
    varargout{1}  = data.timeStamps         ;
    varargout{2}  = data.tracked            ;
    varargout{3}  = data.calibrated         ;
    varargout{4}  = data.pupilDiameter      ;
    varargout{5}  = data.mmPositions        ;
    varargout{6}  = data.fickPositions      ;
    varargout{7}  = data.helmholtzPositions ;
    varargout{8}  = data.region             ;
    varargout{9}  = data.fixation           ;
    varargout{10} = data.digitalIO          ;
    varargout{11} = data.dropped            ;
% Undocumented (legacy) usage - only returns monocular data
elseif nargout ==  12
    varargout{1}  = data.timeStamps         ;
    varargout{2}  = data.tracked            ;
    varargout{3}  = data.calibrated         ;
    varargout{4}  = data.pupilDiameter      ;
    varargout{5}  = data.mmPositions        ;
    varargout{6}  = data.fickPositions      ;
    varargout{7}  = data.helmholtzPositions ;
    varargout{8}  = data.region             ;
    varargout{9}  = data.fixation           ;
    varargout{10} = data.digitalIO          ;
    varargout{11} = data.dropped            ;
    varargout{13} = data.ErrorCode          ;
else
    error('Unrecognised number of output (LHS) arguments.');
end