function [pupilL, glintL, pupilR, glintR, targets, order] = crsLiveTrackGetFixationDataRaw(viewDist, targetsDeg, WindowID, trackLeftEye, trackRightEye, window)
%CRSLIVETRACKGETFIXATIONDATARAW  This demo shows how raw data from CRS
% LiveTrack eye trackers can be collected for a number of fixations.
%
%   This demo uses Psychtoolbox functions to drive the display while 
%   acquiring data from CRS LiveTrack devices. The script places dots on
%   the screen at various positions while the participant must fixate the
%   each dot until the a valid data has been aquired. If no valid data as
%   been aquired after a time out period, the fixation point will be
%   skipped and the script moves to the next fixation.
%
%   The function will output the average location of the pupil and glint
%   for each fixation in coordinated of the camera image sensor of the eye
%   tracker. The corresponding location of the fixaxtion dots is returned
%   in pixel coordinates of the screen. An array with the order in which
%   the fixations were taken is also returned.
%
%   The parameters for the locations of the fixaxtion dots and the fixation
%   criteria can be adjusted in the "User input" section at the beginning
%   of the function.
%
%   The input to this function must be:
%   viewDist - perpendicular distance between the participants eyes and the
%   screen in millimetres.
%   WindowID - a two-entry vector containing the ID of the control monitor 
%   and the stimulus monitor as specified by the function Screen('Screens'). 


% History:
% 09/2016 JT Created
% 05/2017 MAH adapted to use CRS Toolbox for data 
% 06/2017 JT changed to use buffered data from the CRS Toolbox

%% Default values for input parameters (if omitted)

if nargin<6
    window = [];
end

if nargin<4
    trackLeftEye = [];
    trackRightEye = [];
end

% Define default values for the monitors to use 
if nargin<3
    % Get the IDs of all monitors present
    screensAvaiable = Screen('Screens');

   % ID of experimenter's monitor and stimulus monitor
    WindowID(1) = max(screensAvaiable);
    if length(screensAvaiable)>1
        WindowID(2) = max(screensAvaiable)-1;       
    end
end

% Define the defualt location of the the fixation targets in degrees
% (origin at the centre, x-positive=right, y-positive=down)
if nargin<2
    targetsDeg = [-10 -10;0 -10;10 -10;-10 0;0 0;10 0;-10 10;0 10;10 10];
end

% Define default value for the viewing distance in mm
if nargin<1
    viewDist = 500;
end

%% User input

% Time before getting data from fixation (to ignore initial saccade)
setupDelay = 500; % duration in mS

% Define the minimum amount of time required for each fixation (in mS) - 
% decrease to make calibration "easier"/faster
minDur = 1200; % fixation duration in mS

% Time limit for aquiring fixation - if a fixation has not been aquired 
% within this time, the fixation point will be skipped.
fixTimeout = 5; % timeout duration in seconds

% Fixation threshold in camera pixels (all samples taken wihtin the "minDur"
% time must be within this threshold) - increase this to make the  
% calibration "easier" (e.g. for subjects with difficulties making precise
% eye fixations)
fixThreshold = 3.1;

% Define the diameter of the fixation points (in degrees of visual angle)
fixDotInDeg = 0.3; % inner circle
fixDotOutDeg = 0.6; % outer circle

% Randomize the order of the targets:
order = randperm(size(targetsDeg,1));
targetsDeg = targetsDeg(order,:);

% Show window with fixation information if more than one monitor is 
% configured and if SkipSyncTests is not set (can cause it to hang when
% using DrawText)
if length(WindowID)>1 && ~Screen('Preference', 'SkipSyncTests')
    showPGwindow=true;
else
    showPGwindow=false;
end

%% Setup LiveTrack

% Start LiveTrack raw data streaming
crsLiveTrackSetResultsTypeRaw;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Get an estimate of the sample rate
[ width, height, sampleRate, offsetX, offsetY, ErrorCode ] = crsLiveTrackGetCaptureConfig; %#ok<ASGLU>

% Calculate how many data samples the fixation duration (fixDur) contains
fixDurSamples = round((minDur/1000)*sampleRate);

% Find out which eye to get fixation data from
if isempty(trackLeftEye) || isempty(trackRightEye)
    [trackLeftEye, trackRightEye] = crsLiveTrackGetTracking;
end

%% Calculate stimulus screen specification

% Use default settings for Psychtoolbox
if isempty(window)
    PsychDefaultSetup(2);
end

% Get the resolution of the stimulus monitor
ResStim = Screen('Resolution', WindowID(1));

% Get screen width and height (in mm)
screenW = Screen('DisplaySize', WindowID(1));

% Calculate pixel size (in mm) assuming square pixels
pixSize = screenW/ResStim.width;

% Caliculate millimetres per degree (NB. Only accurate for small eye
% angles)
MMperDeg = tand(1)*viewDist;

% Calculate pixels per degree
pixPerDeg = MMperDeg/pixSize;

% Calculate the radius of the fixation points (in pixels)
fixRadIn = fixDotInDeg*pixPerDeg/2;
fixRadOut = fixDotOutDeg*pixPerDeg/2;

% Target locations in screen pixel coordinates
cnrTarget = [round(ResStim.width/2) round(ResStim.height/2)];
tgtLocs(:,1) = round(targetsDeg(:,1)*pixPerDeg+cnrTarget(1)); 
tgtLocs(:,2) = round(targetsDeg(:,2)*pixPerDeg+cnrTarget(2));

%% Open PTB windows and start collecting data

% Open a full screen window with grey background on stimulus monitor
if isempty(window)
    winStim = PsychImaging('OpenWindow', WindowID(1), 0.5);
else
    winStim = window;
end

% wait a second so first dot is not shown immediately after window opens
pause(1);

% Open a small window on the experimenter's monitor to show some 
% information about the subject's fixation performance
if showPGwindow
    winResExp=[640 360];
    ResExp=Screen('Resolution', WindowID(2));
    win1 = PsychImaging('OpenWindow', WindowID(2), 0.5, [ResExp.width/2-winResExp(1)/2 ResExp.height/2-winResExp(2)/2 ResExp.width/2+winResExp(1)/2 ResExp.height/2+winResExp(2)/2]);
    Screen('TextSize', win1, 10);
end

% Initialise the output data
pupilL = nan(size(targetsDeg,1),2);
pupilR = nan(size(targetsDeg,1),2);
glintL = nan(size(targetsDeg,1),2);
glintR = nan(size(targetsDeg,1),2);
targets = nan(size(targetsDeg,1),2);
EscPressed = false;

% Run a loop over each fixation point and get the fixation data
for i=1:size(targetsDeg,1)
    
    % Draw new fixation point with the outer circle at 75% grey and the
    % inner circle black
    Screen('FillOval', winStim, 0.75, round([tgtLocs(i,1)-fixRadOut...
        tgtLocs(i,2)-fixRadOut tgtLocs(i,1)+fixRadOut tgtLocs(i,2)+fixRadOut]));
    Screen('FillOval', winStim, 0, round([tgtLocs(i,1)-fixRadIn...
        tgtLocs(i,2)-fixRadIn tgtLocs(i,1)+fixRadIn tgtLocs(i,2)+fixRadIn]));
    Screen('Flip', winStim);

    % This flag will be set to true when a valid fixation has been acquired
    gotFixLeft = false;  
    gotFixRight = false;
       
    tic; % reset fixation timer 
    % Loop until fixation data has been aquired for this dot (or timed out) 
    while 1

        % Make sure there is at least one sample in the buffer before
        % continuing
        while crsLiveTrackGetResultsCount<1
            if toc>1
                error('ERROR: Could not get any data from LiveTrack!');
            end
        end
        
        % Get the most recent data samples since the buffer was cleared,
        % up to the number specified in fixDurSamples
        Data = crsLiveTrackGetLatestEyePosition(fixDurSamples);

        % Calculate the pupil-to-glint vector for x and y direction and
        % left and right eye
        PGvectorLeftX = Data.pupilPositions(:,1)-Data.glintPositions(:,1);
        PGvectorLeftY = Data.pupilPositions(:,2)-Data.glintPositions(:,2);
        PGvectorRightX = Data.pupilPositions(:,1)-Data.glintPositions(:,1);
        PGvectorRightY = Data.pupilPositionsRight(:,2)-Data.glintPositionsRight(:,2);

        % Calculate the maximum difference in the pupil-to-glint 
        % vectors for the samples in the buffer, for left eye
        pgDistLeft = max([max(PGvectorLeftX)-min(PGvectorLeftX) ...
            max(PGvectorLeftY)-min(PGvectorLeftY)]);
        % and for the right eye
        pgDistRight = max([max(PGvectorRightX)-min(PGvectorRightX) ...
            max(PGvectorRightY)-min(PGvectorRightY)]);

        % Check if the maximum vector difference is within the defined
        % limit for a fixation (fixWindow) and all samples are tracked, and
        % the time to wait for fixations (waitTimeForFix) has passed, for
        % the left eye
        if pgDistLeft<=fixThreshold && all(Data.tracked) && toc>setupDelay/1000
            % Check if there are enough samples in the buffer for the
            % defined duration (fixDurSamples)
            if size(PGvectorLeftX,1)>=fixDurSamples && ~gotFixLeft
                % save the data for this fixation
                pupilL(i,:) = median(Data.pupilPositions);
                glintL(i,:) = median(Data.glintPositions);
                disp(['Fixation #',num2str(i),': Found valid fixation for left eye']);
                gotFixLeft = true; % good fixation aquired
            end
        end
        % and for the right eye
        if pgDistRight<=fixThreshold && all(Data.trackedRight) && toc>setupDelay/1000
            % Check if there are enough samples in the buffer for the
            % defined duration (fixDurSamples)
            if size(PGvectorRightX,1)>=fixDurSamples && ~gotFixRight
                % save the data for this fixation
                pupilR(i,:) = median(Data.pupilPositionsRight);
                glintR(i,:) = median(Data.glintPositionsRight);
                disp(['Fixation #',num2str(i),': Found valid fixation for right eye']);
                gotFixRight = true; % good fixation aquired
            end
        end
          
        if showPGwindow
            showPGvector(win1,Data,winResExp,pgDistLeft,pgDistRight,fixThreshold,PGvectorLeftX,PGvectorLeftY,PGvectorRightX,PGvectorRightY,trackLeftEye,trackRightEye);
        end
        [keyIsDown,~,keyCode]=KbCheck;
        if keyIsDown
            if keyCode(KbName('ESCAPE'))
                sca;
                crsLiveTrackStopTracking;
                crsLiveTrackClearDataBuffer;
                crsLiveTrackClose;
                error('Esc pressed');
            end
            if keyCode(KbName('p'))
                disp('paused');
                pause(0.5); % debounce
                KbWait;
                pause(0.5); % debounce
                disp('unpaused');
            end
        end
        
        if toc>fixTimeout
            if ~gotFixLeft && trackLeftEye
                disp(['Fixation #',num2str(i),': Did not get fixation for left eye (timeout)']);
            end
            if ~gotFixRight && trackRightEye
                disp(['Fixation #',num2str(i),': Did not get fixation for right eye (timeout)']);
            end
            break; % fixation timed out
        end
        
        % Exit if all eyes that are enabled have got a fixation
        if (gotFixLeft || ~trackLeftEye) && (gotFixRight || ~trackRightEye)
            Screen('Flip', winStim);
            break;
        end
    end
                 
    % Return the target positions in millimetres
    targets(i,:) = targetsDeg(i,:)*MMperDeg;
    
    if EscPressed
        break
    end

end

Screen('Flip', winStim);

if isempty(window)
    disp('closing PTB');
    % Close all PTB windows
    sca;
end

% Stop buffering data to the library
crsLiveTrackStopTracking;

% Clear the data in the buffer
crsLiveTrackClearDataBuffer;

% put data back in order before returning it
pupilL(order,:) = pupilL;
glintL(order,:) = glintL;
pupilR(order,:) = pupilR;
glintR(order,:) = glintR;
targets(order,:) = targets;

% function to update debug window
function showPGvector(winExp,Data,winResExp,pgDistLeft,pgDistRight,fixWindow,PGvectorLeftX,PGvectorLeftY,PGvectorRightX,PGvectorRightY,trackLeftEye,trackRightEye)

    % draw a frame around the data area for left and right eyes
    if trackLeftEye, Screen('FrameRect', winExp,[1 1 1],[10 10 winResExp(1)/2-10 winResExp(1)/2-10]); end
    if trackRightEye, Screen('FrameRect', winExp,[1 1 1],[winResExp(1)/2+10 10 winResExp(1)-10 winResExp(1)/2-10]); end
        
    % Write title for each frame
    if trackLeftEye, DrawFormattedText(winExp,'Left eye PG vector',15, 20, 1); end
    if trackRightEye, DrawFormattedText(winExp,'Right eye PG vector',winResExp(1)/2+15, 20, 1); end
    
    % draw a square with a side length relative to the fixation criteria
    % (fixWindow; the pupil-to-glint vectors in the buffer must within 
    % this window) for left and right eyes
    Cr = winResExp(1)/16; % radius of fixWindow in pixels
    if trackLeftEye, Screen('FrameRect', winExp,[1 1 1],[winResExp(1)/4-Cr winResExp(1)/4-Cr winResExp(1)/4+Cr winResExp(1)/4+Cr]); end
    if trackRightEye, Screen('FrameRect', winExp,[1 1 1],[3*winResExp(1)/4-Cr winResExp(1)/4-Cr 3*winResExp(1)/4+Cr winResExp(1)/4+Cr]); end
     
    clear xy
    xy(1,:) = winResExp(1)/4+((PGvectorLeftX-mean([max(PGvectorLeftX) min(PGvectorLeftX)]))/fixWindow)*Cr*2;
    xy(1,xy(1,:)<12) = 13;
    xy(1,xy(1,:)>winResExp(1)/2-12) = winResExp(1)/2-12;
    xy(2,:) = winResExp(1)/4+((PGvectorLeftY-mean([max(PGvectorLeftY) min(PGvectorLeftY)]))/fixWindow)*Cr*2; 
    xy(2,xy(2,:)<12) = 12;
    xy(2,xy(2,:)>winResExp(1)/2-12) = winResExp(1)/2-12;
    if trackLeftEye, Screen('DrawLines', winExp, xy); end
    if trackLeftEye, Screen('DrawLines', winExp, xy(:,2:end)); end
    clear xy
    xy(1,:) = 3*winResExp(1)/4+((PGvectorRightX-mean([max(PGvectorRightX) min(PGvectorRightX)]))/fixWindow)*Cr*2;
    xy(1,xy(1,:)<winResExp(1)/2+13) = winResExp(1)/2+13;
    xy(1,xy(1,:)>winResExp(1)-12) = winResExp(1)-12;
    xy(2,:) = winResExp(1)/4+((PGvectorRightY-mean([max(PGvectorRightY) min(PGvectorRightY)]))/fixWindow)*Cr*2; 
    xy(2,xy(2,:)<12) = 12;
    xy(2,xy(2,:)>winResExp(1)/2-12) = winResExp(1)/2-12;
    if trackRightEye, Screen('DrawLines', winExp, xy); end
    if trackRightEye, Screen('DrawLines', winExp, xy(:,2:end)); end
          
    % write the current gaze error(s) and the error limit
    if trackLeftEye, DrawFormattedText(winExp,['Left Delta Gaze=',num2str(pgDistLeft)],15, winResExp(1)/2, 1); end
    if trackRightEye, DrawFormattedText(winExp,['Right Delta Gaze=',num2str(pgDistRight)],winResExp(1)/2+15, winResExp(1)/2, 1); end
    
    % Write the current time stamp and fixation window size
    DrawFormattedText(winExp,['Timestamp = ',num2str(round(Data.timeStamps(end)/10^6)),' S'], 15, winResExp(1)/2+20, 1);
    DrawFormattedText(winExp,['Fixation window size set to = ',num2str(fixWindow)], 15, winResExp(1)/2+30, 1);
    
    
    % Draw warnings if eye is not tracked, for the left eye
    if ~Data.tracked(end) && trackLeftEye
        DrawFormattedText(winExp,'Left eye not tracked',15, winResExp(1)/2+10, [1 0 0]);
    end
    % and for the right eye
    if ~Data.trackedRight(end) && trackRightEye
        DrawFormattedText(winExp,'Right eye not tracked',winResExp(1)/2+15, winResExp(1)/2+10, [1 0 0]);
    end
    
    Screen('Flip', winExp);

end

end