function [LGaze, RGaze, targets] = crsLiveTrackGetFixationDataCalibrated(viewDist, targetsDeg, WindowID)
%CRSLIVETRACKGETFIXATIONDATACALIBRATED This demo shows how calibrated data
% from CRS LiveTrack eye trackers can be collected for a number of fixations.
%
%   USAGE:
%   [LGaze, RGaze, targets] = crsLiveTrackGetFixationDataCalibrated(viewDist) 
%   sets the viewing distance in millimeters (default is 500 mm) and
%   returns the gaze vector in millimetres for left and right eye
%   The gaze vector consists of three columns: X, Y and Z, where X and Y
%   are the coordinates in the plane of the screen with origin at the
%   centre of the screen and Z is the viewing distance. The targets
%   parameter consists of the target location on the screen im mm.
%
% History:
% 06/2017 JT Created


%% Default values for input parameters (if omitted)

% Define default values for the monitors to use 
if nargin<3
    % Get the IDs of all monitors present
    screensAvaiable = Screen('Screens');

    % Use secondary monitor
    WindowID = max(screensAvaiable);
end

% Define the defualt location of the the fixation targets in degrees
% (origin at the centre, x-positive=right, y-positive=down)
if nargin<2
    targetsDeg = [-10 -10;0 -10;10 -10;-10 0;0 0;10 0;-10 10;0 10;10 10];
end

% Define default value for the viewing distance in mm
if nargin<1
    viewDist = 500;
end

%% User input

% Time before getting data from fixation (to ignore initial saccade)
setupDelay = 500; % duration in mS

% Define the minimum amount of time required for each fixation (in mS) - 
% decrease to make calibration "easier"/faster
minDur = 1000; % fixation duration in mS

% Time limit for aquiring fixation - if a fixation has not been aquired 
% within this time, the fixation point will be skipped.
fixTimeout = 5; % timeout duration in seconds

% Fixation threshold in camera pixels (all samples taken wihtin the "minDur"
% time must be within this threshold) - increase this to make the  
% calibration "easier" (e.g. for subjects with difficulties making precise
% eye fixations)
fixThreshold = 3.1;

% Define the diameter of the fixation points (in degrees of visual angle)
fixDotInDeg = 0.3; % inner circle
fixDotOutDeg = 0.6; % outer circle

% Randomize the order of the targets:
order = randperm(size(targetsDeg,1));
targetsDeg = targetsDeg(order,:);

%% 

% Find out which eye to get fixation data from
[trackLeftEye, trackRightEye] = crsLiveTrackGetTracking;


%% Setup LiveTrack

% Start streaming calibrated results
crsLiveTrackSetResultsTypeCalibrated;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Get the sample rate
[ width, height, sampleRate, offsetX, offsetY, ErrorCode ] = crsLiveTrackGetCaptureConfig; %#ok<ASGLU>

% Calculate how many data samples the fixation duration (fixDur) contains
fixDurSamples = round((minDur/1000)*sampleRate);

%% Calculate stimulus screen specification

% Use default settings for Psychtoolbox
PsychDefaultSetup(2);

% Get the resolution of the stimulus monitor
Res = Screen('Resolution', WindowID);

% Get screen width and height (in mm)
screenW = Screen('DisplaySize', WindowID);

% Calculate pixel size (in mm) assuming square pixels
pixSize = screenW/Res.width;

% Caliculate millimetres per degree
MMperDeg = tand(1)*viewDist;

% Calculate pixels per degree
pixPerDeg = MMperDeg/pixSize;

% Calculate the radius of the fixation points (in pixels)
fixRadIn = fixDotInDeg*pixPerDeg/2;
fixRadOut = fixDotOutDeg*pixPerDeg/2;

% Target locations in screen pixel coordinates
cnrTarget = [round(Res.width/2) round(Res.height/2)];
tgtLocs(:,1) = round(targetsDeg(:,1)*pixPerDeg+cnrTarget(1)); 
tgtLocs(:,2) = round(targetsDeg(:,2)*pixPerDeg+cnrTarget(2));


%% Open PTB windows and start collecting data

% Open a full screen window with grey background on stimulus monitor
win = PsychImaging('OpenWindow', WindowID, 0.5);

% Initialise the output data
LGaze = nan(size(targetsDeg,1),3);
RGaze = nan(size(targetsDeg,1),3);
targets = nan(size(targetsDeg,1),2);

% Run a loop over each fixation point and get the fixation data
for i=1:size(targetsDeg,1)
    
    % Draw new fixation point with the outer circle at 75% grey and the
    % inner circle black
    Screen('FillOval', win, 0.75, round([tgtLocs(i,1)-fixRadOut...
        tgtLocs(i,2)-fixRadOut tgtLocs(i,1)+fixRadOut tgtLocs(i,2)+fixRadOut]));
    Screen('FillOval', win, 0, round([tgtLocs(i,1)-fixRadIn...
        tgtLocs(i,2)-fixRadIn tgtLocs(i,1)+fixRadIn tgtLocs(i,2)+fixRadIn]));
    Screen('Flip', win);
    
    % This flag will be set to true when a valid fixation has been acquired
    gotFixLeft = false;  
    gotFixRight = false;
       
    tic; % reset fixation timer 
    % Loop until fixation data has been aquired for this dot (or timed out) 
    while 1

        % Make sure there is at lease one sample in the buffer before
        % continuing
        while crsLiveTrackGetResultsCount<1
            if toc>1
                error('ERROR: Could not get any data. Is Livetrack initiated?');
            end
        end
        
        % Get the most recent data samples since the buffer was cleared,
        % up to the number specified in fixDurSamples
        Data = crsLiveTrackGetLatestEyePosition(fixDurSamples);

        % Calculate the maximum difference in angular eye position for
        % left and right eye
        DiffLeft = max([max(Data.helmholtzPositions(:,1))-min(Data.helmholtzPositions(:,1)) ...
            max(Data.helmholtzPositions(:,2))-min(Data.helmholtzPositions(:,2))]);
        DiffRight = max([max(Data.helmholtzPositionsRight(:,1))-min(Data.helmholtzPositionsRight(:,1)) ...
            max(Data.helmholtzPositionsRight(:,2))-min(Data.helmholtzPositionsRight(:,2))]);
        
        % Check if the maximum angular eye position is within the defined
        % limit for a fixation (fixWindow) and all samples are tracked, and
        % the time to wait for fixations (waitTimeForFix) has passed, for
        % the left eye
        if DiffLeft<=fixThreshold && all(Data.tracked) && toc>setupDelay/1000
            % Check if there are enough samples in the buffer for the
            % defined duration (fixDurSamples)
            if size(Data.tracked,1)>=fixDurSamples && ~gotFixLeft
                % save the data for this fixation
                LGaze(i,:) = median(Data.mmPositions);
                disp(['Fixation #',num2str(i),': Found valid fixation for left eye']);
                gotFixLeft = true; % good fixation aquired
            end
        end
        % and for the right eye
        if DiffRight<=fixThreshold && all(Data.trackedRight) && toc>setupDelay/1000
            % Check if there are enough samples in the buffer for the
            % defined duration (fixDurSamples)
            if size(Data.trackedRight,1)>=fixDurSamples && ~gotFixRight
                % save the data for this fixation
                RGaze(i,:) = median(Data.mmPositionsRight);
                disp(['Fixation #',num2str(i),': Found valid fixation for right eye']);
                gotFixRight = true; % good fixation aquired
            end
        end
          
        [keyIsDown,~,keyCode]=KbCheck;
        if keyIsDown
            if keyCode(KbName('ESCAPE'))
                Screen('CloseAll');
                disp('Esc pressed');
                error('Esc pressed');
            end
        end
        
        if toc>fixTimeout
            if ~gotFixLeft && trackLeftEye
                disp(['Fixation #',num2str(i),': Did not get fixation for left eye (timeout)']);
                disp(['Fixation #',num2str(i),': Last data in the buffer was within ',num2str(DiffLeft),...
                    ' but the limit was ',num2str(fixThreshold)]);
                disp('Try increaseing the limit (fixWindow)');
            end
            if ~gotFixRight && trackRightEye
                disp(['Fixation #',num2str(i),': Did not get fixation for right eye (timeout)']);
                disp(['Fixation #',num2str(i),': Last data in the buffer was within ',num2str(DiffRight),...
                    ' but the limit was ',num2str(fixThreshold)]);
                disp('Try increaseing the limit (fixWindow)');
            end
            break; % fixation timed out
        end
        
        % Exit if all eyes that are enabled have got a fixation
        if (gotFixLeft || ~trackLeftEye) && (gotFixRight || ~trackRightEye)
            Screen('Flip', win);
            break;
        end
    end
                 
    % Return the target positions in millimetres
    targets(i,:) = targetsDeg(i,:)*MMperDeg;

end

% Close all PTB windows
sca;

% Stop buffering data to the library
crsLiveTrackStopTracking;

% Clear the data in the buffer
crsLiveTrackClearDataBuffer;

% put data back in order before returning it
LGaze(order,:) = LGaze;
RGaze(order,:) = RGaze;
targets(order,:) = targets;
